package com.example.demo.supply.Order;

import com.example.demo.supply.Product.Product;
import com.example.demo.supply.Supplier.Supplier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Date;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
public class OrderService {

    private final OrderRepository orderRepository;

    public OrderService(OrderRepository orderRepository){
        this.orderRepository = orderRepository;
    }

    @Transactional
    public Orders addOrder(Supplier supplier){
        final Orders order = new Orders(new Date(System.currentTimeMillis()));
        order.setSupplier(supplier);
        return orderRepository.save(order);
    }

    //поставщики, у которых есть заказ на конкретный товар или несколько товаров
    @Transactional
    public List<Supplier> suppliers(List<Product> products){
        List<Supplier> result = new ArrayList<>();
        List<Orders> orders = findAllOrders();
        for(Orders order : orders){
            int k = 0;
            for(Product product : products){
                if(order.getProducts().contains(product)) k++;
            }
            if(k == products.size())
                result.add(order.getSupplier());
        }
        return result;
    }

    @Transactional
    public Orders addProduct(Long id, Product product) {
        final Orders currentOrder = findOrder(id);
        currentOrder.addProduct(product);
        return orderRepository.save(currentOrder);
    }

    @Transactional(readOnly = true)
    public Orders findOrder(Long id) {
        final Optional<Orders> order = orderRepository.findById(id);
        return order.orElseThrow(() -> new OrderNotFoundException(id));
    }

    @Transactional(readOnly = true)
    public List<Orders> findAllOrders() {
        return orderRepository.findAll();
    }

    @Transactional
    public Orders updateOrder(Long id, Date date) {
        final Orders currentOrder = findOrder(id);
        currentOrder.setDateOfOrder(date);
        return orderRepository.save(currentOrder);
    }

    @Transactional
    public Orders deleteOrder(Long id) {
        final Orders currentOrder = findOrder(id);
        orderRepository.delete(currentOrder);
        return currentOrder;
    }

    @Transactional
    public void deleteAll(){
        orderRepository.deleteAll();
    }
}
